<?php

// Фреймворк ArangoDB
use mirzaev\arangodb\connection,
	mirzaev\arangodb\collection,
	mirzaev\arangodb\document;

// Библиотека для ArangoDB
use ArangoDBClient\Document as _document,
	ArangoDBClient\Cursor,
	ArangoDBClient\Statement as _statement;

// Фреймворк Telegram
use Zanzara\Zanzara,
	Zanzara\Context,
	Zanzara\Config,
	Zanzara\Telegram\Type\Input\InputFile,
	Zanzara\Telegram\Type\File\Document as telegram_document,
	Zanzara\Telegram\Type\File\File,
	Zanzara\Middleware\MiddlewareNode as Node;

ini_set('error_reporting', E_ALL);
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);

// Версия робота
define('VERSION', '1.0.0');

// Путь до настроек
define('SETTINGS', __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'settings');

// Путь до хранилища
define('STORAGE', __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'storage');

// Файл в формате xlsx с примером excel-документа для импорта каталога
define('CATALOG_EXAMPLE', STORAGE . DIRECTORY_SEPARATOR . 'example.xlsx');

// Файл в формате xlsx для импорта каталога
define('CATALOG_IMPORT', STORAGE . DIRECTORY_SEPARATOR . 'import.xlsx');

// Ключ чат-робота Telegram
define('KEY', require(SETTINGS . DIRECTORY_SEPARATOR . 'key.php'));

// Инициализация библиотек
require __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php';

// Инициализация инстанции соединения с ArangoDB
$arangodb = new connection(require SETTINGS . DIRECTORY_SEPARATOR . 'arangodb.php');

/**
 * Экранирование символов для Markdown
 *
 * @param string $text Текст для экранирования
 * @param array $exception Символы которые будут исключены из списка для экранирования
 *
 * @return string Экранированный текст
 */
function unmarkdown(string $text, array $exceptions = []): string
{
	// Инициализация реестра символом для конвертации
	$from = array_diff(
		[
			'#',
			'*',
			'_',
			'=',
			'.',
			'[',
			']',
			'(',
			')',
			'-',
			'>',
			'<',
			'!',
			'`'
		],
		$exceptions
	);

	// Инициализация реестра целей для конвертации
	$to = [];
	foreach ($from as $symbol) $to[] = "\\$symbol";

	// Конвертация и выход (успех)
	return str_replace($from, $to,	$text);
}

/**
 * Инициализация запчасти
 *
 * Проверяет существование запчасти
 *
 * @param string $spare Запчасть
 *
 * @return string|bool Запчасть, если найдена, иначе false
 */
function spares(string $spare): string|bool
{
	// Поиск запчастей и выход (успех)
	return match (mb_strtolower($spare)) {
		'цевьё' => 'Цевьё',
		default => false
	};
}

/**
 * Авторизация
 *
 * @param string $id Идентификатор Telegram
 * @param bool $registration Регистрация, если аккаунт не найден
 *
 * @return _document|null Инстанция аккаунта, если найден
 */
function authorization(string $id, bool $registration = true): _document|null
{
	global $arangodb;

	if (collection::init($arangodb->session, 'account')) {
		if ($account = collection::search($arangodb->session, sprintf("FOR d IN account FILTER d.id == '%s' RETURN d", $id))) {
			// Найден аккаунт

			// Возврат (успех)
			return $account;
		} else if ($registration) {
			// Не найден аккаунт и запрошена его регистрация

			// Создание аккаунта
			document::write($arangodb->session,	'account', ['id' => $id, 'banned' => false, 'settings' => false, 'version' => VERSION]);

			// Авторизация (без регистрации)
			return authorization($id, false);
		} else {
			// Не найден аккаунт и не запрошена его регистрация

			// Выход (провал)
			return null;
		}
	} else throw new exception('Не удалось инициализировать коллекцию: account');

	// Выход (провал)
	return null;
}


/**
 * Главное меню
 *
 * Команда: /start
 *
 * @param Context $ctx
 *
 * @return void
 */
function menu(Context $ctx): void
{
	// Инициализация клавиатуры
	$keyboard = [
		[
			['text' => '🛒 Каталог']
		],
		[
			['text' => '💬 Контакты'],
			['text' => '🏛️ О компании']
		],
		[
			['text' => '🎯 Сообщество']
		]
	];

	if ($ctx->get('account')?->settings) $keyboard[] = [['text' => '⚙️  Настройки']];

	// Отправка сообщения
	$ctx->sendMessage(
		unmarkdown(<<<TXT
		Это сообщение будет отображаться (оно должно быть обязательно) при вызове главного меню командой /start (создаёт кнопки меню снизу)
		TXT),
		[
			'reply_markup' => [
				'keyboard' => $keyboard,
				'resize_keyboard' => true
			],
			'disable_notification' => true
		]
	);
}

/**
 * Категории
 *
 * Команда: /catalog
 *
 * @param Context $ctx
 *
 * @return void
 */
function categories(Context $ctx): void
{
	// Отправка сообщения
	$ctx->sendMessage(unmarkdown(<<<TXT
		Выберите категорию
		TXT), [
		'reply_markup' => [
			'inline_keyboard' => [
				[
					['text' => '🗜️ Тюнинг', 'callback_data' => 'tuning']
				]
			]
		],
		'link_preview_options' => [
			'is_disabled' => true
		],
		'disable_notification' => true
	]);
}

/**
 * Тюнинг
 *
 * @param Context $ctx
 *
 * @return void
 */
function tuning(Context $ctx): void
{
	// Отправка сообщения
	$ctx->sendMessage(<<<TXT
		Выберите запчасть
		TXT, [
		'reply_markup' => [
			'inline_keyboard' => [
				[
					['text' => 'Цевьё', 'callback_data' => 'brands']
				]
			]
		],
		'link_preview_options' => [
			'is_disabled' => true
		],
		'disable_notification' => true
	]);
}

/**
 * Бренды
 *
 * @param Context $ctx
 *
 * @return void
 */
function brands(Context $ctx): void
{
	if ($spare = spares($ctx->getMessage()->getText())) {
		// Инициализирована запчасть 

		// Отправка сообщения
		$ctx->sendMessage(<<<TXT
		Выберите бренд
		TXT, [
			'link_preview_options' => [
				'is_disabled' => true
			],
			'disable_notification' => true
		])->then(function ($message) use ($ctx) {
			$ctx;
		});
	} else {
		// Не инициализирована запчасть

		// Отправка сообщения
		$ctx->sendMessage('⚠️ *Не найдена запчасть*');
	}
}

/**
 * Контакты
 *
 * Команда: /contacts
 *
 * @param Context $ctx
 *
 * @return void
 */
function contacts(Context $ctx): void
{
	// Отправка сообщения
	$ctx->sendMessage(unmarkdown(<<<TXT
		Здесь придумать текст для раздела "Контакты"
		TXT), [
		'reply_markup' => [
			'inline_keyboard' => [
				[
					['text' => '⚡ Связь с менеджером', 'url' => 'https://t.me/iarming'],
				],
				[
					['text' => '📨 Почта', 'callback_data' => 'mail']
				],
				[
					['text' => '🪖 Сайт', 'url' => 'https://arming.ru'],
					['text' => '🛒 Wildberries', 'url' => 'https://arming.ru']
				]
			]
		],
		'link_preview_options' => [
			'is_disabled' => true
		],
		'disable_notification' => true
	]);
}

/**
 * Почта
 *
 * @param Context $ctx
 *
 * @return void
 */
function _mail(Context $ctx): void
{
	// Отправка сообщения
	$ctx->sendMessage(unmarkdown(<<<TXT
		[info@arming.ru](mailto::info@arming.ru)
		TXT, ['[', ']', '(', ')']), [
		'link_preview_options' => [
			'is_disabled' => true
		],
		'disable_notification' => true
	]);
}

/**
 * Компания
 *
 * Команда: /company
 *
 * @param Context $ctx
 *
 * @return void
 */
function company(Context $ctx): void
{
	// Отправка сообщения
	$ctx->sendMessage(
		unmarkdown(<<<TXT
		Здесь придумать текст для раздела "Компания"
		TXT),
		/* [
		'reply_markup' => [
			'inline_keyboard' => [
				[
					['text' => '⚡ Связь с менеджером', 'url' => 'https://git.mirzaev.sexy/mirzaev/mashtrash'],
					['text' => '📨 Почта', 'text' => ''],
				],
				[
					['text' => '🪖 Сайт', 'url' => '']
					['text' => '🛒 Wildberries', 'url' => '']
				]
			]
		],
		'link_preview_options' => [
			'is_disabled' => true
		]
	] */
	);
}

/**
 * Сообщество
 *
 * Команда: /community
 *
 * @param Context $ctx
 *
 * @return void
 */
function community(Context $ctx): void
{
	// Отправка сообщения
	$ctx->sendMessage(unmarkdown(<<<TXT
		Здесь придумать текст для раздела "Сообщество"
		TXT), [
		'reply_markup' => [
			'inline_keyboard' => [
				[
					['text' => '💬 Основной чат', 'url' => 'https://t.me/arming_zone'],
				]
			]
		],
		'link_preview_options' => [
			'is_disabled' => true
		],
		'disable_notification' => true
	]);
}

/**
 * Настройки (доступ только авторизованным)
 *
 * Команда: /settings
 *
 * @param Context $ctx
 *
 * @return void
 */
function settings(Context $ctx): void
{
	if ($ctx->get('account')?->settings) {
		// Авторизован доступ к настройкам

		// Отправка сообщения
		$ctx->sendMessage(
			unmarkdown(<<<TXT
			Панель управления чат-роботом ARMING
			TXT),
			[
				'reply_markup' => [
					'inline_keyboard' => [
						[
							['text' => '📦 Импорт товаров', 'callback_data' => 'import_request'],
						]
					]
				],
				'link_preview_options' => [
					'is_disabled' => true
				],
				'disable_notification' => true
			]
		);
	} else {
		// Не авторизован доступ к настройкам

		// Отправка сообщения
		$ctx->sendMessage('⛔ *Нет доступа*');
	}
}

/**
 * Запросить файл для импорта товаров (доступ только авторизованным)
 *
 * @param Context $ctx
 *
 * @return void
 */
function import_request(Context $ctx): void
{
	if ($ctx->get('account')?->settings) {
		// Авторизован доступ к настройкам

		// Отправка сообщения
		$ctx->sendMessage(unmarkdown('Отправьте документ в формате xlsx со списком товаров'))
			->then(function ($message) use ($ctx) {
				// Отправка файла
				$ctx->sendDocument(new InputFile(CATALOG_EXAMPLE), ['disable_notification' => true]);

				// Импорт файла
				$ctx->nextStep("import");
			});
	} else {
		// Не авторизован доступ к настройкам

		// Отправка сообщения
		$ctx->sendMessage('⛔ *Нет доступа*');
	}
}

/**
 * Импорт товаров (доступ только авторизованным)
 *
 * @param Context $ctx
 *
 * @return void
 */
function import(Context $ctx): void
{
	if ($ctx->get('account')?->settings) {
		// Авторизован доступ к настройкам

		// Инициализация документа
		$document = $ctx->getMessage()?->getDocument();

		if ($document instanceof telegram_document) {
			// Инициализирован документ

			// Инициализация файла
			$ctx->getFile($document->getFileId())->then(function ($file) use ($ctx) {

				if ($file->getFileSize() <= 50000000) {
					// Не превышает 50 мегабайт (50000000 байт) размер файла

					if ($file->getFilePath()['extension'] === 'xlsx') {
						// Имеет расширение xlsx файл

						// Сохранение файла
						file_put_contents(STORAGE . DIRECTORY_SEPARATOR . 'import.xlsx', file_get_contents('https://api.telegram.org/file/bot' . KEY . '/' . $file->getFilePath()));

						// Инициализация счётчика загруженных товаров
						$loaded = $created = $updated = $deleted = $old = $new = 0;



						// Отправка сообщения
						$ctx->sendMessage(<<<TXT
							*Загружено для обработки:* $loaded

							*Добавлено:* $created
							*Обновлено:* $updated
							*Удалено:* $deleted

							*Опубликовано в магазине:* $old \-\> *$new*
							TXT)
							->then(function ($message) use ($ctx) {
								// Завершение диалога
								$ctx->endConversation();
							});
					} else {
						// Не имеет расширение xlsx файл

						// Отправка сообщения
						$ctx->sendMessage(unmarkdown('Файл должен иметь расширение xlsx'));
					}
				} else {
					// Превышает 50 мегабайт (50000000 байт) размер файла

					// Отправка сообщения
					$ctx->sendMessage(unmarkdown('Размер файла не должен превышать 50 мегабайт'));
				}
			});
		} else {
			// Не инициализирован документ

			// Отправка сообщения
			$ctx->sendMessage(unmarkdown('Отправьте документ в формате xlsx со списком товаров'));
		}
	} else {
		// Не авторизован доступ к настройкам

		// Отправка сообщения
		$ctx->sendMessage('⛔ *Нет доступа*');
	}
}

$config = new Config();
$config->setParseMode(Config::PARSE_MODE_MARKDOWN);
$config->useReactFileSystem(true);

$bot = new Zanzara(KEY, $config);
$bot->onCommand('start', function (Context $ctx): void {
	// Главное меню
	menu($ctx);
});

/* $bot->onUpdate(function (Context $ctx): void {}); */

/**
 * Инициализация аккаунта (middleware)
 *
 * @param Context $ctx
 * @param Node $next
 *
 * @return void
 */
$account = function (Context $ctx, Node $next): void {
	// Выполнение заблокировано?
	if ($ctx->get('stop')) return;

	// Авторизация аккаунта
	$account = authorization($ctx->getEffectiveUser()->getId());

	if ($account instanceof _document) {
		// Инициализирован аккаунт (подразумевается)

		if ($account->banned) {
			// Заблокирован аккаунт

			// Отправка сообщения
			$ctx->sendMessage('⛔ *Ты заблокирован*');

			// Завершение диалога
			$ctx->endConversation();

			// Блокировка дальнейшего выполнения
			$ctx->set('stop', true);
		} else {
			// Не заблокирован аккаунт

			// Запись в буфер
			$ctx->set('account', $account);

			// Продолжение выполнения
			$next($ctx);
		}
	} else {
		// Не инициализирован аккаунт
	}
};


$bot->onCommand('catalog', fn($ctx) => categories($ctx));
$bot->onCommand('contacts', fn($ctx) => contacts($ctx));
$bot->onCommand('company', fn($ctx) => company($ctx));
$bot->onCommand('community', fn($ctx) => community($ctx));
$bot->onCommand('settings', fn($ctx) => settings($ctx));

$bot->onText('🛒 Каталог', fn($ctx) => categories($ctx));
$bot->onText('💬 Контакты', fn($ctx) => contacts($ctx));
$bot->onText('🏛️ О компании', fn($ctx) => company($ctx));
$bot->onText('🎯 Сообщество', fn($ctx) => community($ctx));
$bot->onText('⚙️  Настройки', fn($ctx) => settings($ctx));

$bot->onCbQueryData(['mail'], fn($ctx) => _mail($ctx));
$bot->onCbQueryData(['import_request'], fn($ctx) => import_request($ctx));
$bot->onCbQueryData(['tuning'], fn($ctx) => tuning($ctx));
$bot->onCbQueryData(['brands'], fn($ctx) => brands($ctx));

$bot->middleware($account)->run();
